<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class Admin_MainController extends Controller
{
    /**
    * set access privileges
    */    
    function init()
    {        
        $this->acl->allow('administrator', $this->name, "*");
        $this->acl->allow('guest', $this->name, "logIn");
        $this->acl->allow('guest', $this->name, "lostPassword");
         
        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect(FrontController::getRewrittedUrl("/admin/main/logIn"));  
        }
    }   
    
    /**
    * Display admin index page
    */
    function indexAction()
    {           
        $this->set("databaseSize", $this->database->getTotalSize());

        $this->set("waitingSitesCount", $this->statistic->getCountOfSitesWithStatus("waiting"));
        $this->set("validatedSitesCount", $this->statistic->getCountOfSitesWithStatus("validated"));
        $this->set("refusedSitesCount", $this->statistic->countOfAllRefusals());
        
        $c = new Criteria();
        $c->add("role", "webmaster");
        $this->set("webmastersCount", $this->user->getCount($c));
        
        $this->set("categoriesCount", $this->category->getCount());
        $this->set("keywordsCount", $this->keyword->getCount());
        require(CODE_ROOT_DIR."config/version.php");
        $this->set("directoryVersion", $currentVersion);
    }
    
    /**
    * Display login form and login admin
    */
    function logInAction()
    {
        if(isset($this->request->login) && isset($this->request->password))
        {            
            if($this->session->loginUser($this->request->login, md5($this->request->password), "login", "administrator"))
            {                
                $httpClient = new HttpClient();
                $this->session->set("scriptLastVersion", $httpClient->getSiteContent("http://script.arfooo.com/version/version.html", false));                              
                $this->redirect($this->moduleLink("index"));
            }
            else
            {
                $this->set("loginError", 1);
            }
        }     
    }
    
    /**
    * Logout admin
    */
    function logOutAction()
    {
        $this->session->destroy();
        $this->redirect($this->moduleLink());  
    }
    
    /**
    * Display lost password form and send it to user email
    */
    function lostPasswordAction()
    {
        //if no posta data only display form
        if(empty($this->request->email))return;
        
        //search that email is correct
        $c = new Criteria();
        $c->add("email", $this->request->email);
        $c->add("role", "administrator");
        $user = $this->user->find($c);
        
        if(empty($user))return;

        //generate and send new password
        $newPassword = $user->generateNewPassword();

        Mailer::getInstance()->sendLostPassword($user->email, $newPassword);

        $this->redirect($this->moduleLink("logIn"));
    }
    
    /**
    * Clear directory for make data fresh
    */
    private function clearDir($path)
    {                                            
        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::CHILD_FIRST);
        
        foreach($iterator as $file)
        {
            if(basename($file->getFilename()) == ".htaccess")continue;
            
            if($file->isDir())
            {
                rmdir($file->getPathname());
            }
            else
            {
                unlink($file->getPathname());
            }
        }     
    }
    
    /**
    * Clear actions
    */
    function clearAction()
    {
        $cmd = "";
        
        foreach(array("topHits", "topNotes", "topReferrers", "fileCache", "googleCache") as $opt)
        {
            if(!empty($_POST[$opt]))$cmd = $opt;
        }
        
        switch($cmd)
        {
            case "topHits":
                $this->hit->removeAll();
                break; 
            
            case "topNotes":
                $this->vote->removeAll();
                break; 
                
            case "topReferrers":
                $this->otherReferrerSite->removeAll();
                break; 
            
            case "fileCache":
                $cache = Cacher::getInstance();
                $cache->clean("all");
                $this->clearDir(CODE_ROOT_DIR."compiled/");
                break; 
            
            case "googleCache":
                $this->cacheGoogleDetail->removeAll();
                break; 
        }
                                     
        $this->redirect($this->moduleLink());
    }
} 

?>