<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

abstract class BackgroundTask extends Model
{
    protected $taskId;
    protected $items;
    protected $data;
    protected $parsedItems = 0;
    protected $totalItems;
    protected $pauseUpdateStatusInterval;
    protected $request;
    
    abstract protected function loadItems();
    abstract protected function parseItem($item);
    
    protected function beforeInit(){}
    protected function beforeParsing(){}
    protected function afterParsing(){}
    
    public function init()
    {
        $this->request = Request::getInstance(); 
        $this->beforeInit();
        $this->task->delByPk($this->taskId);
        
        $task = new TaskRecord();
        $task->taskId = $this->taskId;
        $task->parsedItems = 0;
        $task->data = serialize($this->data);
        $task->status = "init";
        $task->save();    
    }
    
    public function fork($path)
    {
        $postData = "taskId=".$this->taskId;
        
        $fp = fsockopen("localhost", 80);
        $query = "POST ".$path." HTTP/1.0\r\n"
                    ."Host: ".$_SERVER['HTTP_HOST']."\r\n"
                    ."Content-Type: application/x-www-form-urlencoded\r\n"
                    ."Content-Length: ".strlen($postData)."\r\n"
                    ."Connection: close\r\n\r\n".$postData;

        fwrite($fp, $query);
        
        fclose($fp);  
    }
    
    public function start()
    {
        ignore_user_abort(true);
        set_time_limit(60*60);
        
        $taskCriteria = new Criteria();
        $taskCriteria->add("taskId", $this->taskId);
        
        $task = $this->task->find($taskCriteria);
        
        if(empty($task) || $task->status != "init")return;
        
        $this->loadItems();
        $this->totalItems = count($this->items);
        $this->data = unserialize($task->data);
        
        $task->totalItems = $this->totalItems;
        $task->status = "active";
        $task->save(); 
        $lp = 0;
        
        $this->beforeParsing();

        foreach($this->items as $item)
        {
            $this->parseItem($item);
            
            $task->parsedItems = ++$this->parsedItems;
            $task->save();
            
            if($this->parsedItems < $this->totalItems)
            {
                $taskStatus = $this->task->get("status", $taskCriteria);

                if($taskStatus == "pause")
                {
                    do
                    {
                       sleep($this->pauseUpdateStatusInterval);
                       $taskStatus = $this->task->get("status", $taskCriteria);
                    }while($taskStatus == "pause");
                }
                
                if($taskStatus == "stop")
                {
                    break;
                }  
            }

        }
        
        $task->status = "finish";
        $task->save();   
        
        $this->afterParsing(); 
    }
    
    private function setStatus($status)
    {
        $this->task->updateByPk(array("status" => $status), $this->taskId);
    }

    public function pause()
    {
        return $this->setStatus("pause");
    }
    
    public function resume()
    {
        return $this->setStatus("active");
    }
    
    public function stop()
    {
        return $this->setStatus("stop");
    }

    public function getStatus()
    {
        $task = $this->task->findByPk($this->taskId, "status, parsedItems, totalItems"); 
        return ($task) ? $task->toArray() : array();
    }
}
        
?>